
class BaseVariantMixin(object):
    
    def add_variant(self, variant):
        """Add a variant to the database
        
        Args:
            variant (dict): A variant dictionary
        """
        raise NotImplementedError
    
    def get_variant(self, variant):
        """Return a variant from the database
        
        Args:
            variant (dict): A variant dictionary
    
        Returns:
            variant (dict): A variant dictionary
        """
        raise NotImplementedError
    
    def delete_variant(self, variant):
        """Remove variant from database
            
            This means that we take down the observations variable with one.
            If 'observations' == 1 we remove the variant. If variant was homozygote
            we decrease 'homozygote' with one.
            
            Args:
                variant (dict): A variant dictionary            
        """
        raise NotImplementedError

    def add_bulk(self, variants):
        """Add a bulk of variants to the database
        
        Args:
            variants (Iterable(dict)): An iterable with variant dictionaries
        """
        raise NotImplementedError
    


from loqusdb.plugins import MongoAdapter

class TestInsertVariant:
    
    def test_insert_one_variant(self, mongo_adapter, simplest_variant):
        """Test to insert one variant"""
        
        mongo_adapter.add_variant(simplest_variant)
        
        db = mongo_adapter.db
        
        #Get the variant without the adapter
        mongo_variant = db.variant.find_one()
        
        assert mongo_variant['_id'] == 'test'
        assert mongo_variant['observations'] == 1
        assert mongo_variant['homozygote'] == 0
    
    def test_insert_one_variant_twice(self, mongo_adapter, simplest_variant):
        """Test to insert one variant"""
    
        mongo_adapter.add_variant(simplest_variant)
        mongo_adapter.add_variant(simplest_variant)
        
        db = mongo_adapter.db

        mongo_variant = db.variant.find_one()

        assert mongo_variant['_id'] == 'test'
        assert mongo_variant['observations'] == 2
        assert mongo_variant.get('homozygote',0) == 0
    
    def test_insert_hom_variant(self, mongo_adapter, homozygous_variant):
        """Test to insert a homozygote variant"""
        
        mongo_adapter.add_variant(homozygous_variant)

        db = mongo_adapter.db
    
        mongo_variant = db.variant.find_one()
        assert mongo_variant['_id'] == 'test'
        assert mongo_variant['observations'] == 1
        assert mongo_variant.get('homozygote', 0) == 1
        assert mongo_variant['families'] == ['1']

    def test_insert_many(self, mongo_adapter, simplest_variant):
        """Test to insert a homozygote variant"""
        
        for i in range(10000):
            mongo_adapter.add_variant(simplest_variant)

        db = mongo_adapter.db
    
        mongo_variant = db.variant.find_one()
        assert mongo_variant['_id'] == 'test'
        assert mongo_variant['observations'] == 10000
        assert mongo_variant.get('homozygote', 0) == 0


class TestGetVariant:
    def test_get_variant(self, mongo_client, mongo_adapter, simplest_variant):
        """Test to insert one variant"""
        
        #Insert without adapter
        db = mongo_client['test']
        db.variant.insert(simplest_variant)
        
        mongo_variant = mongo_adapter.get_variant(simplest_variant)
        assert mongo_variant['_id'] == 'test'
    
    def test_get_none(self, mongo_adapter, simplest_variant):
        """Test to get non existing variant"""
    
        mongo_variant = mongo_adapter.get_variant(simplest_variant)
    
        assert mongo_variant == None
        
#These tests does not work with mongomock yet so we skip them for now
# class TestBulkOperations:
#
#     def test_insert_one_variant(self, real_mongo_client, simplest_variant):
#         """Test to insert one variant with bulk insert"""
#
#         adapter = MongoAdapter()
#         adapter.connect(database='test', client=real_mongo_client)
#
#         variants = [simplest_variant]
#
#         adapter.add_bulk(variants)
#         db = adapter.db
#         mongo_variant = db.variant.find_one()
#
#         assert mongo_variant['_id'] == 'test'
#         assert mongo_variant['observations'] == 1
#         assert mongo_variant['homozygote'] == 0
#
#     def test_insert_two_variants(self, real_mongo_client):
#         """Test to insert two variants with bulk"""
#
#         adapter = MongoAdapter()
#         adapter.connect(database='test', client=real_mongo_client)
#         db = adapter.db
#
#         variants = []
#         variants.append({
#             '_id': 'test',
#             'homozygote': 0
#         })
#         variants.append({
#             '_id': 'test_1',
#             'homozygote': 1
#         })
#
#
#         adapter.add_bulk(variants)
#
#         first_variant = db.variant.find_one({'_id': 'test'})
#         second_variant = db.variant.find_one({'_id': 'test_1'})
#
#         assert first_variant['_id'] == 'test'
#         assert first_variant['observations'] == 1
#         assert first_variant.get('homozygote',0) == 0
#
#         assert second_variant['_id'] == 'test_1'
#         assert second_variant['observations'] == 1
#         assert second_variant.get('homozygote',0) == 1
#
#
#     def test_insert_many(self, real_mongo_client):
#
#         adapter = MongoAdapter()
#         adapter.connect(database='test', client=real_mongo_client)
#         db = adapter.db
#
#         variants = ({'_id': 'test','homozygote': 0} for i in range(10000))
#
#         adapter.add_bulk(variants)
#
#         mongo_variant = db.variant.find_one()
#
#         assert mongo_variant['_id'] == 'test'
#         assert mongo_variant['observations'] == 10000
#         assert mongo_variant.get('homozygote', 0) == 0

class TestRemoveVariant:
    
    def test_remove_one_variant(self, mongo_adapter):
        """Test to update one variant"""
    
        db = mongo_adapter.db
    
        variant = {
            '_id': 'test',
            'observations': 1
        }
    
        db.variant.insert(variant)
    
        mongo_adapter.delete_variant(variant)

        assert db.variant.find_one() == None
    
    def test_downcount_one_variant(self, mongo_adapter):
        """Test to update one variant"""
    
        db = mongo_adapter.db
    
        insert_variant = {
            '_id': 'test',
            'families':['1', '2'],
            'observations': 2
        }
    
        db.variant.insert(insert_variant)

        variant = {
            '_id': 'test',
            'family_id':'1'
        }
        
        mongo_adapter.delete_variant(variant)
    
        mongo_variant = db.variant.find_one()

        assert mongo_variant['observations'] == 1
        assert mongo_variant['families'] == ['2']
    
    def test_remove_non_existing(self, mongo_adapter, simplest_variant):
    
        db = mongo_adapter.db
    
        mongo_adapter.delete_variant(simplest_variant)

        assert db.variant.find_one() == None
    



import os
import logging
import click

from loqusdb.exceptions import CaseError
from loqusdb.utils import load_database

from . import base_command

logger = logging.getLogger(__name__)


@base_command.command()
@click.argument('variant_file',
                    type=click.Path(exists=True),
                    metavar='<vcf_file>'
)
@click.option('-f', '--family_file',
                    type=click.Path(exists=True),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                show_default=True,
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.option('-s' ,'--skip_case_id', 
                is_flag=True, 
                show_default=True,
                help='Do not store which cases that have a variant'
)
@click.option('--gq-treshold',
    default=20,
    show_default=True,
    help='Treshold to consider variant'
)
@click.pass_context
def load(ctx, variant_file, family_file, family_type, skip_case_id, gq_treshold):
    """Load the variants of a case

    The loading is based on if the variant is seen in a ny affected individual
    in the family.
    """
    if not family_file:
        logger.warning("Please provide a family file")
        ctx.abort()

    variant_path = os.path.abspath(variant_file)

    adapter = ctx.obj['adapter']
    
    try:
        load_database(
            adapter=adapter,
            variant_file=variant_path,
            family_file=family_file,
            family_type=family_type,
            skip_case_id=skip_case_id,
            gq_treshold=gq_treshold
        )
    except (SyntaxError, CaseError, IOError) as error:
        logger.warning(error.message)
        ctx.abort()
        


import logging
from pymongo import MongoClient

from loqusdb.plugins import Base
from . import VariantMixin, CaseMixin

logger = logging.getLogger(__name__)


class MongoAdapter(VariantMixin, CaseMixin, Base):
    """docstring for MongoAdapter"""
    def __init__(self, debug=False):
        super(MongoAdapter, self).__init__()
        self.db = None
        self.client = None
        self.db_name = None
    
    def connect(self, host='localhost', port=27017, uri=None, 
                database='loqusdb', client=None):
        """Connect to a mongo database
        
        Args:
            host(str) : The host for the mongo datavase
            port(int) : The port for the mongo datavase
            database(str) : The name of the mongo databse
            client(MongoClient) : For testing only
        """
        logger.info("Accessing host:{0}, port:{1}, database:{2}".format(
            host, port, database
        ))
        if not client:
            if uri:
                logger.info("Connecting with uri {0}".format(uri))
                client=MongoClient(uri)
            else:
                client = MongoClient(host, port)
        
        self.client = client
        self.db_name = database
        self.db = client[database]
    
    def init_app(self, app):
        """Initialize via flask app"""
        uri = app.config.get('LOQUSDB_URI')
        self.connect(uri=uri)
    
    def wipe_db(self):
        """Wipe the whole database"""
        logger.warning("Wiping the whole database")
        self.client.drop_database(self.db_name)
        logger.debug("Database wiped")
        



import pytest

from loqusdb.utils import load_database
from loqusdb.exceptions import CaseError

def test_load_database(vcf_path, ped_path, mongo_adapter, case_id):
    db = mongo_adapter.db
    
    load_database(
        adapter=mongo_adapter, 
        variant_file=vcf_path, 
        family_file=ped_path, 
        family_type='ped', 
    )
    
    mongo_case = db.case.find_one()
    
    assert mongo_case['case_id'] == case_id

def test_load_database_wrong_ped(vcf_path, funny_ped_path, mongo_adapter):
    db = mongo_adapter.db
    
    with pytest.raises(CaseError):
        load_database(
            adapter=mongo_adapter, 
            variant_file=vcf_path, 
            family_file=funny_ped_path, 
            family_type='ped', 
        )
    



import os
import sys
import logging

LEVELS = {
    1 : 'INFO',
    2 : 'DEBUG',
}

def init_log(logger, filename=None, loglevel=None):
    """
    Initializes the log file in the proper format.

    Arguments:

        filename (str): Path to a file. Or None if logging is to
                         be disabled.
        loglevel (str): Determines the level of the log output.
    """
    template = '[%(asctime)s] %(levelname)-5s: %(name)-30s: %(message)s'
    formatter = logging.Formatter(template)

    if loglevel:
        logger.setLevel(getattr(logging, loglevel))

    # We will always print warnings and higher to stderr
    console = logging.StreamHandler()
    console.setLevel('WARNING')
    console.setFormatter(formatter)

    if filename:
        file_handler = logging.FileHandler(filename, encoding='utf-8')
        if loglevel:
            file_handler.setLevel(getattr(logging, loglevel))
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    # If no logfile is provided we print all log messages that the user has
    # defined to stderr
    if loglevel:
        console.setLevel(getattr(logging, loglevel))

    logger.addHandler(console)

def get_log_stream(logger):
    """
    Returns a stream to the root log file.
    If there is no logfile return the stderr log stream

    Returns:
        A stream to the root log file or stderr stream.
    """

    file_stream = None
    log_stream = None
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            file_stream = handler.stream
        else:
            log_stream = handler.stream

    if file_stream:
        return file_stream

    return log_stream



import pytest

from loqusdb.exceptions import CaseError

class TestDeleteCase:
    
    def test_delete_one_case(self, mongo_adapter, simple_case):
        """Test to delete one case"""
    
        db = mongo_adapter.db
    
        mongo_case = db.case.find_one()
        
        assert mongo_case == None
        
        db.case.insert(simple_case)
    
        mongo_case = db.case.find_one()
    
        assert mongo_case['case_id'] == simple_case['case_id']
        assert mongo_case['vcf_path'] == simple_case['vcf_path']
    
        mongo_adapter.delete_case(simple_case)

        assert db.case.find_one() == None
    

    def test_delete_non_existing(self, mongo_adapter, simple_case):
        """Test to delete non exsting case"""
        
        with pytest.raises(CaseError):
            mongo_adapter.delete_case(simple_case)

class TestInsertCase:
    
    def test_insert_one_case(self, mongo_adapter, simple_case):
    
        db = mongo_adapter.db
        
        mongo_adapter.add_case(simple_case)
        
        mongo_case = db.case.find_one()
    
        assert mongo_case['case_id'] == simple_case['case_id']
        assert mongo_case['vcf_path'] == simple_case['vcf_path']

    def test_insert_one_case_twice(self, mongo_adapter, simple_case):
    
        db = mongo_adapter.db
        
        mongo_adapter.add_case(simple_case)
        
        with pytest.raises(CaseError):
            mongo_adapter.add_case(simple_case)
        

class TestGetCase:

    def test_get_case(self, mongo_adapter, simple_case):
        """Test to get non existing case"""
    
        db = mongo_adapter.db
        db.case.insert(simple_case)

        mongo_case = mongo_adapter.case(simple_case)
    
        assert mongo_case['case_id'] == simple_case['case_id']
        assert mongo_case['vcf_path'] == simple_case['vcf_path']
        
        
    def test_get_non_existing(self, mongo_adapter, simple_case):
        """Test to get non existing case"""
    
        db = mongo_adapter.db

        mongo_case = mongo_adapter.case(simple_case)
    
        assert mongo_case == None
    
    def test_get_multiple_cases(self, mongo_adapter):
        """Test to get non existing case"""
    
        db = mongo_adapter.db
        case_1 = {
            'case_id': 'test',
            'vcf_path': 'test.vcf'
        }

        case_2 = {
            'case_id': 'test2',
            'vcf_path': 'test2.vcf'
        }
        
        db.case.insert(case_1)
        db.case.insert(case_2)

        mongo_cases = mongo_adapter.cases()

        assert mongo_cases.count() == 2



import logging
import pytest
from mongomock import MongoClient
from pymongo import MongoClient as RealMongoClient

from ped_parser import FamilyParser

from loqusdb.plugins import MongoAdapter
from loqusdb.log import init_log

logger = logging.getLogger('.')

init_log(logger, loglevel='DEBUG')

@pytest.fixture(scope='function')
def mongo_client(request):
    """Return a mongomock client"""
    client = MongoClient()
    return client

# @pytest.fixture(scope='function')
# def real_mongo_client(request):
#     """Return a mongo client"""
#     client = RealMongoClient()
#
#     def teardown():
#         client.drop_database('test')
#
#     request.addfinalizer(teardown)
#     return client
#
# @pytest.fixture(scope='function')
# def real_mongo_adapter(request, real_mongo_client):
#     """Return a mongo adapter"""
#     adapter = MongoAdapter()
#     adapter.connect(database='test', client=real_mongo_client)
#     return adapter

@pytest.fixture(scope='function')
def mongo_adapter(request, mongo_client):
    """Return a mongo adapter"""
    adapter = MongoAdapter()
    adapter.connect(database='test', client=mongo_client)
    return adapter

@pytest.fixture(scope='function')
def simplest_variant(request):
    variant = {
        '_id': 'test',
    }
    return variant

@pytest.fixture(scope='function')
def homozygous_variant(request):
    variant = {
        '_id': 'test',
        'homozygote': 1,
        'family_id': '1'
    }
    return variant

@pytest.fixture(scope='function')
def simple_case(request):
    case = {
        'case_id': 'test',
        'vcf_path': './test.vcf'
    }
    return case

@pytest.fixture(scope='function')
def vcf_path(request):
    file_path = 'tests/fixtures/test.vcf'
    return file_path

@pytest.fixture(scope='function')
def zipped_vcf_path(request):
    file_path = 'tests/fixtures/test.vcf.gz'
    return file_path

@pytest.fixture(scope='function')
def ped_path(request):
    file_path = 'tests/fixtures/recessive_trio.ped'
    return file_path

@pytest.fixture(scope='function')
def funny_ped_path(request):
    file_path = 'tests/fixtures/funny_trio.ped'
    return file_path

def get_variant(variant_line, header):
    """docstring for get_variant"""
    return dict(zip(header, variant_line.split('\t')))

def get_header(inds=['proband', 'mother', 'father']):
    """docstring for header"""
    header = ['CHROM', 'POS', 'ID', 'REF', 'ALT', 'QUAL', 'FILTER', 'INFO', 
              'FORMAT']
    for ind in inds:
        header.append(ind)
    
    return header

def variant_line(chrom='1', pos='10', rs_id='.', ref='A', alt='T', 
                 qual='100', filt='PASS', info='.', form='GT:GQ', 
                 genotypes=['0/1:60','0/1:60','0/0:60']):
    """Return a vcf formated variant line"""
    variant_line = "{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}".format(
        chrom, pos, rs_id, ref, alt, qual, filt, info, form
    )
    
    for gt_call in genotypes:
        variant_line += '\t{0}'.format(gt_call)
    
    return variant_line

@pytest.fixture(scope='function')
def case_lines(request, ped_path):
    """Return ped formated case lines"""
    case = []
    with open(ped_path, 'r') as f:
        for line in f:
            case.append(line) 
    
    return case

@pytest.fixture(scope='function')
def case_obj(request, case_lines):
    """Return a case obj"""
    family_parser = FamilyParser(case_lines, family_type='ped')
    
    families = list(family_parser.families.keys())
    
    family = family_parser.families[families[0]]

    return family

@pytest.fixture(scope='function')
def case_id(request, case_lines):
    """Return a case obj"""
    family_parser = FamilyParser(case_lines, family_type='ped')
    
    families = list(family_parser.families.keys())
    
    family = family_parser.families[families[0]]
    
    family_id = family.family_id

    return family_id

@pytest.fixture(scope='function')
def individuals(request, case_obj):
    """Return a case obj"""
    
    return case_obj.individuals

@pytest.fixture(scope='function')
def two_cases(request):
    """Return ped formated case lines"""
    case_lines = [
        "#FamilyID\tSampleID\tFather\tMother\tSex\tPhenotype\n",
        "1\tproband\t0\t0\t1\t2\n",
        "2\tproband\t0\t0\t2\t1\n",
    ]
    return case_lines

@pytest.fixture(scope='function')
def hem_variant(request):
    variant = variant_line(chrom='X', pos='60000')
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object

@pytest.fixture(scope='function')
def par_variant(request):
    variant = variant_line(chrom='X', pos='60001')
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object

@pytest.fixture(scope='function')
def het_variant(request):
    variant = variant_line()
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object

@pytest.fixture(scope='function')
def variant_no_gq(request):
    variant = variant_line(form='GT', genotypes=['0/1','0/1','0/1'])
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object

@pytest.fixture(scope='function')
def hom_variant(request):
    variant = variant_line(genotypes=['1/1:60','0/1:60','0/1:60'])
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object

@pytest.fixture(scope='function')
def variant_no_call(request):
    variant = variant_line(genotypes=['./.', './.', './.'], form='GT')
    header = get_header()
    variant_object = get_variant(variant, header)
    return variant_object



import logging

from loqusdb.plugins import BaseVariantMixin

logger = logging.getLogger(__name__)

class VariantMixin(BaseVariantMixin):
    
    def add_variant(self, variant):
        """Add a variant to the variant collection
        
            If the variant exists we update the count else we insert a new variant
            object.
        
            Args:
                variant (dict): A variant dictionary
        
        """
        logger.debug("Upserting variant: {0}".format(variant.get('_id')))
        
        if variant.get('family_id'):
            message = self.db.variant.update(
                {'_id': variant['_id']},
                {
                    '$inc': {
                        'homozygote': variant.get('homozygote', 0),
                        'hemizygote': variant.get('hemizygote', 0),
                        'observations': 1
                    },
                    '$push': {
                        'families': {
                            '$each': [variant.get('family_id')],
                            '$slice': -50
                            }
                        }
                 }, 
                 upsert=True
            )
        else:
            message = self.db.variant.update(
                {'_id': variant['_id']},
                {
                    '$inc': {
                        'homozygote': variant.get('homozygote', 0),
                        'hemizygote': variant.get('hemizygote', 0),
                        'observations': 1,
                    }
                 }, 
                 upsert=True
            )
            
    
        if message.get('updatedExisting'):
            logger.debug("Variant {0} was updated".format(message.get("upserted")))
        else:
            logger.debug("Variant was added to database for first time")
        return
    
    def get_variant(self, variant):
        """Check if a variant exists in the database and return it
    
            Search the variants with the variant id
        
            Args:
                variant (dict): A variant dictionary
        
            Returns:
                variant (dict): A variant dictionary
        """
        return self.db.variant.find_one({'_id': variant.get('_id')})

    def get_variants(self):
        """Return all variants in the database
    
            Returns:
                variants (Iterable(Variant))
        """
        return self.db.variant.find()

    def delete_variant(self, variant):
        """Remove variant from database
            
            This means that we take down the observations variable with one.
            If 'observations' == 1 we remove the variant. If variant was homozygote
            we decrease 'homozygote' with one.
            
            Args:
                variant (dict): A variant dictionary            
        """
        mongo_variant = self.get_variant(variant)
        
        if mongo_variant:
            
            if mongo_variant['observations'] == 1:
                logger.debug("Removing variant {0}".format(
                    mongo_variant.get('_id')
                ))
                message = self.db.variant.remove({'_id': variant['_id']})
            else:
                logger.debug("Decreasing observations for {0}".format(
                    mongo_variant.get('_id')
                ))
                message = self.db.variant.update({
                    '_id': mongo_variant['_id']
                    },{
                        '$inc': {
                            'observations': -1,
                            'homozygote': - (variant.get('homozygote', 0)),
                            'hemizygote': - (variant.get('hemizygote', 0)),
                        },
                        '$pull': {
                            'families': variant.get('family_id')
                        }
                    }, upsert=False)
        return



import logging
import click

from . import base_command

logger = logging.getLogger(__name__)

def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()


@base_command.command()
@click.option('--yes',
    is_flag=True,
    callback=abort_if_false,
    expose_value=False,
    prompt='Are you sure you want to wipe the entire database?'
)
@click.pass_context
def wipe(ctx):
    """Wipe the entire db
    
    """
    adapter = ctx.obj['adapter']
    
    adapter.wipe_db()
    



# -*- coding: utf-8 -*-
from datetime import datetime
import logging

from loqusdb.vcf_tools import (get_formated_variant, get_vcf)
from loqusdb.utils import (get_family)
from loqusdb.exceptions import CaseError

logger = logging.getLogger(__name__)

def load_database(adapter, variant_file, family_file, family_type='ped',
                  skip_case_id=False, gq_treshold=None):
    """Load the database with a case and its variants
            
            Args:
                  adapter
                  variant_file(str)
                  family_file(str)
                  family_type(str)
                  skip_case_id(bool)
 
    """

    vcf = get_vcf(variant_file)
    
    with open(family_file, 'r') as family_lines:
        family = get_family(
            family_lines=family_lines, 
            family_type=family_type
        )

    family_id = family.family_id

    if not family.affected_individuals:
        logger.warning("No affected individuals could be found in ped file")
    
    logger.info("Found affected individuals in ped file: {0}"
                .format(', '.join(family.affected_individuals)))

    load_family(
        adapter=adapter,
        case_id=family_id,
        vcf_path=variant_file
    )

    load_variants(  
        adapter=adapter, 
        family_id=family_id, 
        individuals=family.individuals,
        vcf=vcf, 
        skip_case_id=skip_case_id,
        gq_treshold=gq_treshold,
    )
    

def load_variants(adapter, family_id, individuals, vcf, skip_case_id=False, gq_treshold=None):
    """Load variants for a family into the database.

    Args:
        adapter (loqusdb.plugins.Adapter): initialized plugin
        family_id (str): unique family identifier
        inidividuals (List[str]): list to match individuals
        vcf (iterable(dict)): An iterable variant dictionaries
        skip_case_id (bool): whether to include the case id on variant level 
                             or not
    """
    gq_treshold = gq_treshold or 20
    nr_of_variants = 0
    nr_of_inserted = 0

    start_inserting = datetime.now()
    chrom_time = datetime.now()
    current_chrom = None
    new_chrom = None

    if skip_case_id:
        family_id = None

    # Loop over the variants in the vcf
    for variant in vcf:
            
        nr_of_variants += 1
        
        #Creates a variant that is ready to insert into the database
        formated_variant = get_formated_variant(
                        variant=variant,
                        individuals=individuals,
                        family_id=family_id,
                        gq_treshold=gq_treshold,
                    )

        if formated_variant:
            nr_of_inserted += 1
            adapter.add_variant(variant=formated_variant)
            
            new_chrom = formated_variant.get('chrom')

            if new_chrom != current_chrom:
                if current_chrom:
                    logger.info("Chromosome {0} done".format(current_chrom))
                    logger.info("Time to load chromosome {0}: {1}".format(
                        current_chrom, datetime.now()-chrom_time))
                    logger.info("Start parsing chromosome {0}".format(new_chrom))
                else:
                    logger.info("Start parsing chromosome {}".format(new_chrom))
            
                current_chrom = new_chrom
                chrom_time = datetime.now()

    logger.info("Nr of variants in vcf: {0}".format(nr_of_variants))
    logger.info("Nr of variants inserted: {0}".format(nr_of_inserted))
    logger.info("Time to insert variants: {0}".format(datetime.now() -
                                                      start_inserting))

def load_family(adapter, case_id, vcf_path):
    """Load a case to the database
    
        The adapter will check if the case already exists before loading.
    
        Args:
            adapter (loqusdb.plugins.Adapter): initialized plugin
            case_id (str)
            vcf_path (str)
    """
    case = {'case_id': case_id, 'vcf_path': vcf_path}
    adapter.add_case(case)



import pytest
from loqusdb.utils import (delete_family, load_family)
from loqusdb.exceptions import CaseError

def test_delete_family(mongo_adapter):
    db = mongo_adapter.db
    
    case = {
        'case_id': '1',
        'vcf_path': 'path_to_vcf'
    }
    
    load_family(
        adapter=mongo_adapter,
        case_id=case['case_id'],
        vcf_path=case['vcf_path']
        )
    
    mongo_case = db.case.find_one()
    
    assert mongo_case['case_id'] == case['case_id']
    
    delete_family(
        adapter=mongo_adapter,
        family_id='1',
    )
    
    mongo_case = db.case.find_one()
    
    assert mongo_case == None

def test_delete_non_existing_family(mongo_adapter):
    db = mongo_adapter.db
    
    with pytest.raises(CaseError):
        delete_family(
            adapter=mongo_adapter,
            family_id='1',
        )



import pytest
from loqusdb.utils import load_variants

def test_load_variants(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db
    
    vcf = []
    vcf.append(het_variant)
    
    family_id = case_obj.family_id
    individuals = case_obj.individuals

    mongo_variant = db.variant.find_one()
    
    assert mongo_variant == None
    
    load_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf, 
        gq_treshold=20,
    )
    
    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id]
    assert mongo_variant['observations'] == 1
    assert mongo_variant['homozygote'] == 0
    assert mongo_variant['hemizygote'] == 0

def test_load_homozygote(mongo_adapter, hom_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db
    
    vcf = []
    vcf.append(hom_variant)
    
    family_id = case_obj.family_id
    individuals = case_obj.individuals

    mongo_variant = db.variant.find_one()
    
    assert mongo_variant == None
    
    load_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf, 
        gq_treshold=20,
    )
    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id]
    assert mongo_variant['observations'] == 1
    assert mongo_variant['homozygote'] == 1
    assert mongo_variant['hemizygote'] == 0

def test_load_hemozygote(mongo_adapter, hem_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db
    
    vcf = []
    vcf.append(hem_variant)
    
    family_id = case_obj.family_id
    individuals = case_obj.individuals

    mongo_variant = db.variant.find_one()
    
    assert mongo_variant == None
    
    load_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf, 
        gq_treshold=20,
    )
    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id]
    assert mongo_variant['observations'] == 1
    assert mongo_variant['homozygote'] == 0
    assert mongo_variant['hemizygote'] == 1

def test_load_par_variant(mongo_adapter, par_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db
    
    vcf = []
    vcf.append(par_variant)
    
    family_id = case_obj.family_id
    individuals = case_obj.individuals

    mongo_variant = db.variant.find_one()
    
    assert mongo_variant == None
    
    load_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf, 
        gq_treshold=20,
    )
    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id]
    assert mongo_variant['observations'] == 1
    assert mongo_variant['homozygote'] == 0
    assert mongo_variant['hemizygote'] == 0

def test_load_two_variants(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db

    vcf = []
    vcf.append(het_variant)
    vcf.append(het_variant)

    family_id = case_obj.family_id
    individuals = case_obj.individuals

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id,
        individuals=individuals,
        vcf=vcf,
        gq_treshold=20,
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant['observations'] == 2

def test_load_variants_skip_case_id(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db

    vcf = []
    vcf.append(het_variant)

    family_id = case_obj.family_id
    individuals = case_obj.individuals

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id,
        individuals=individuals,
        vcf=vcf,
        skip_case_id=True,
        gq_treshold=20,
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant.get('families') == None

def test_load_same_variant_different_case(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db

    vcf = []
    vcf.append(het_variant)

    family_id = case_obj.family_id
    individuals = case_obj.individuals

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id,
        individuals=individuals,
        vcf=vcf,
    )

    family_id_2 = '2'
    individuals = case_obj.individuals

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id_2,
        individuals=individuals,
        vcf=vcf,
        gq_treshold=20,
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant['observations'] == 2
    assert mongo_variant['families'] == [family_id, family_id_2]

# # This test works when using a real mongo adapter but not with mongomock yet...
# # def test_load_same_variant_many_cases(real_mongo_adapter, family_variant):
# #     """docstring for test_load_variants"""
# #     db = real_mongo_adapter.db
# #
# #     vcf = []
# #     vcf.append(family_variant)
# #     individuals=['proband', 'mother', 'father']
# #
# #     for i in range(40):
# #         family_id = str(i)
# #
# #         load_variants(
# #             adapter=real_mongo_adapter,
# #             family_id=family_id,
# #             individuals=individuals,
# #             vcf=vcf,
# #             )
# #
# #     mongo_variant = db.variant.find_one()
# #
# #     assert mongo_variant['observations'] == 40
# #     assert len(mongo_variant['families']) == 20


# -*- coding: utf-8 -*-
import logging
from datetime import datetime

from loqusdb.vcf_tools import (get_formated_variant, get_vcf)
from loqusdb.utils import (get_family)

logger = logging.getLogger(__name__)


def delete(adapter, variant_file, family_file, family_type='ped'):
    """Delete a case and all of it's variants from the database"""
    
    vcf = get_vcf(variant_file)

    with open(family_file, 'r') as family_lines:
        family = get_family(
            family_lines=family_lines, 
            family_type=family_type
        )
    
    family_id = family.family_id
    individuals = family.individuals
    
    delete_family(
        adapter=adapter,
        family_id=family_id
    )
    
    delete_variants(
        adapter=adapter,
        vcf=vcf,
        family_id=family_id,
        individuals=individuals
    )

def delete_family(adapter, family_id):
    """Delete a case object from the database"""
    case = {'case_id': family_id}
    adapter.delete_case(case)

def delete_variants(adapter, vcf, family_id, individuals):
    """Delete variants for a case in the database
    
        Args:
            adapter (loqusdb.plugins.Adapter)
            vcf (iterable(dict))
            family_id (str)
        
        Returns:
            nr_of_deleted (int): Number of deleted variants
    """
    nr_of_deleted = 0
    start_deleting = datetime.now()
    chrom_time = datetime.now()
    current_chrom = None
    new_chrom = None
    
    for variant in vcf:
        formated_variant = get_formated_variant(
            variant=variant,
            individuals=individuals,
            family_id=family_id
        )
        
        if formated_variant:
            new_chrom = formated_variant.get('chrom')
            
            adapter.delete_variant(formated_variant)
            nr_of_deleted += 1
            
            if new_chrom != current_chrom:
                if current_chrom:
                    logger.info("Chromosome {0} done".format(current_chrom))
                    logger.info("Time to delete chromosome {0}: {1}".format(
                        current_chrom, datetime.now()-chrom_time))
                    logger.info("Start deleting chromosome {0}".format(new_chrom))
                else:
                    logger.info("Start deleting chromosome {}".format(new_chrom))
            
                current_chrom = new_chrom
                chrom_time = datetime.now()


    return nr_of_deleted



from loqusdb.vcf_tools import get_formated_variant
import pytest


def test_format_variant(het_variant, individuals, case_id):
    formated_variant = get_formated_variant(
        variant=het_variant,
        individuals=individuals,
        family_id=case_id
        )
    
    expected_id = '_'.join([
        het_variant['CHROM'],
        het_variant['POS'], 
        het_variant['REF'], 
        het_variant['ALT']
    ])
    
    assert formated_variant
    assert formated_variant['_id'] == expected_id
    assert formated_variant['chrom'] == het_variant['CHROM']
    assert formated_variant['pos'] == int(het_variant['POS'])
    assert formated_variant['ref'] == het_variant['REF']
    assert formated_variant['alt'] == het_variant['ALT']
    assert formated_variant['family_id'] == case_id
    assert formated_variant['homozygote'] == 0

def test_format_variant_no_gq(variant_no_gq, individuals, case_id):

    formated_variant = get_formated_variant(
        variant=variant_no_gq,
        individuals=individuals,
        family_id=case_id
        )
    assert formated_variant == {}

def test_format_variant_no_family_id(het_variant, individuals):
    formated_variant = get_formated_variant(
        variant=het_variant,
        individuals=individuals,
        family_id=None
        )
    assert formated_variant
    assert formated_variant.get('family_id') == None
    assert formated_variant['homozygote'] == 0
    assert formated_variant['hemizygote'] == 0

def test_format_homozygote_variant(hom_variant, individuals, case_id):

    formated_variant = get_formated_variant(
        variant=hom_variant,
        individuals=individuals,
        family_id=case_id
        )
    assert formated_variant['homozygote'] == 1
    assert formated_variant['hemizygote'] == 0

def test_format_hemizygote_variant(hem_variant, individuals, case_id):

    formated_variant = get_formated_variant(
        variant=hem_variant,
        individuals=individuals,
        family_id=case_id
        )
    assert formated_variant['homozygote'] == 0
    assert formated_variant['hemizygote'] == 1

def test_format_variant_no_call(variant_no_call, individuals, case_id):

    formated_variant = get_formated_variant(
        variant=variant_no_call,
        individuals=individuals,
        family_id=case_id
        )
    assert formated_variant == {}



import logging

from loqusdb.plugins import BaseCaseMixin
from loqusdb.exceptions import CaseError

logger = logging.getLogger(__name__)

class CaseMixin(BaseCaseMixin):
    
    def case(self, case):
        """Get a case from the database
    
            Search the cases with the case id
        
            Args:
                case (dict): A case dictionary
        
            Returns:
                mongo_case (dict): A mongo case dictionary
        """
        logger.debug("Getting case {0} from database".format(
            case.get('case_id')))
        case_id = case['case_id']
        return self.db.case.find_one({'case_id': case_id})

    def cases(self):
        """Get all cases from the database
    
            Returns:
                cases (Iterable(Case)): A iterable with mongo cases
        """
        logger.debug("Getting all casess from database")
        return self.db.case.find()
    
    def add_case(self, case):
        """Add a case to the case collection
        
            If the case exists raise error.
        
            Args:
                db (MongoClient): A connection to the mongodb
                case (dict): A case dictionary
        
        """
        logger.debug("Checking if case {0} exists in database".format(
            case.get('case_id')
        ))
        if self.case(case):
            raise CaseError("Case {0} already exists in database."\
            " Can not add case twice.".format(
                case.get('case_id')
            ))
    
        mongo_case_id = self.db.case.insert_one(case).inserted_id
    
        return mongo_case_id
        
    def delete_case(self, case):
        """Delete case from the database
    
            Delete a case from the database
        
            Args:
                case (dict): A case dictionary
        
        """
        mongo_case = self.case(case)
    
        if mongo_case:
            logger.info("Removing case {0} from database".format(
                mongo_case.get('case_id')
            ))
            self.db.case.remove({'_id': mongo_case['_id']})
        else:
            raise CaseError("Tried to delete case {0} but could not find case".format(
                case.get('case_id')
            ))
        return
    
    def case_count(self):
        """Retursn the total number of cases in the database
        
        returns:
            nr_of_cases (int): Total number of cases in database
        """
        nr_of_cases = 0
        for case in self.cases:
            nr_of_cases += 1
        return nr_of_cases



import os

from codecs import open
import gzip

VALID_ENDINGS = ['.vcf', '.gz']

def get_file_handle(file_path):
    """Return a opened file"""
    if not os.path.exists(file_path):
        raise IOError("No such file:{0}".format(file_path))
    
    if not os.path.splitext(file_path)[-1] in VALID_ENDINGS:
        raise IOError("Not a valid vcf file name: {}".format(file_path))
    
    if file_path.endswith('.gz'):
        file_handle = gzip.open(file_path, 'r')
    else:
        file_handle = open(file_path, 'r')
    
    return file_handle

def get_vcf(file_path):
    """Yield variants from a vcf file
    
        Args:
            file_path(str)
        
        Yields:
            variant(dict): Variant dictionaries

    """

    file_handle = get_file_handle(file_path)

    header = []
    for line in file_handle:
        line = line.rstrip()
        if line.startswith('#'):
            if not line.startswith('##'):
                header = line[1:].split('\t')
        else:
            variant = dict(zip(header, line.split('\t')))
            yield variant



from loqusdb.utils import (load_database, delete)

def test_delete_case_and_variants(vcf_path, ped_path, mongo_adapter, case_id):
    db = mongo_adapter.db

    load_database(
        adapter=mongo_adapter, 
        variant_file=vcf_path, 
        family_file=ped_path, 
        family_type='ped', 
    )

    mongo_case = db.case.find_one()

    assert mongo_case['case_id'] == case_id

    delete(
        adapter=mongo_adapter,
        variant_file=vcf_path,
        family_file=ped_path,
        family_type='ped',
    )

    mongo_case = db.case.find_one()
    
    assert mongo_case == None
    
    mongo_variant = db.case.find_one()
    
    assert mongo_variant == None
    



import pytest

from loqusdb.utils import load_family
from loqusdb.exceptions import CaseError

def test_load_family(mongo_adapter):
    db = mongo_adapter.db
    
    case = {
        'case_id': '1',
        'vcf_path': 'path_to_vcf'
    }
    
    load_family(
        adapter=mongo_adapter,
        case_id=case['case_id'],
        vcf_path=case['vcf_path']
        )
    
    mongo_case = db.case.find_one()
    
    assert mongo_case['case_id'] == case['case_id']


def test_load_same_family_twice(mongo_adapter):
    db = mongo_adapter.db

    case = {
        'case_id': '1',
        'vcf_path': 'path_to_vcf'
    }
    
    load_family(
        adapter=mongo_adapter,
        case_id=case['case_id'],
        vcf_path=case['vcf_path']
        )
    
    with pytest.raises(CaseError):
        load_family(
            adapter=mongo_adapter,
            case_id=case['case_id'],
            vcf_path=case['vcf_path']
            )
        



import pytest
import gzip
from loqusdb.vcf_tools.vcf import get_vcf, get_file_handle

def test_get_file_handle(vcf_path):
    vcf = get_file_handle(vcf_path)
    assert type(vcf) is type(open(vcf_path, 'r'))

def test_get_zipped_file_handle(zipped_vcf_path):
    vcf = get_file_handle(zipped_vcf_path)
    assert type(vcf) is type(gzip.open(zipped_vcf_path, 'r'))

def test_get_vcf_non_vcf(ped_path):
    with pytest.raises(IOError):
        vcf = get_file_handle(ped_path)

def test_get_vcf_non_existing():
    with pytest.raises(IOError):
        vcf = get_file_handle('hello')

def test_get_vcf(vcf_path):
    vcf = get_vcf(vcf_path)
    assert type(vcf) is type((i for i in [1,2]))



from loqusdb.plugins import MongoAdapter

def test_connect(mongo_client):
    adapter = MongoAdapter()
    db_name = 'test'
    adapter.connect(database=db_name, client=mongo_client)
    assert True

def test_connect_uri(mongo_client):
    adapter = MongoAdapter()
    uri = "{0}".format("mongodb://localhost")
    adapter.connect(uri=uri)
    assert True


import pytest

from loqusdb.utils import get_family
from loqusdb.exceptions import CaseError

def test_get_family(case_lines):
    family = get_family(case_lines)
    assert family.family_id == 'recessive_trio'

def test_get_multiple_families(two_cases):
    with pytest.raises(CaseError):
        family = get_family(two_cases)


from loqusdb.vcf_tools.variant import check_par

def test_par_region_X_lower():
    chrom = 'X'
    pos = 60001
    assert check_par(chrom, pos)

def test_par_region_X_middle():
    chrom = 'X'
    pos = 1000000
    assert check_par(chrom, pos)

def test_par_region_X_upper():
    chrom = 'X'
    pos = 2649520
    assert check_par(chrom, pos)

def test_par_region_X_second():
    chrom = 'X'
    pos = 154931044
    assert check_par(chrom, pos)

def test_non_par_X_region():
    chrom = 'X'
    pos = 60000
    assert not check_par(chrom, pos)

def test_par_wrong_chrom():
    chrom = '1'
    pos = 60000
    assert not check_par(chrom, pos)

def test_par_region_Y_lower():
    chrom = 'Y'
    pos = 10001
    assert check_par(chrom, pos)

def test_par_region_Y_middle():
    chrom = 'Y'
    pos = 1000000
    assert check_par(chrom, pos)

def test_par_region_Y_upper():
    chrom = 'Y'
    pos = 2649520
    assert check_par(chrom, pos)

def test_par_region_Y_second():
    chrom = 'Y'
    pos = 59034050
    assert check_par(chrom, pos)



import logging

from ped_parser import FamilyParser

from loqusdb.exceptions import CaseError

logger = logging.getLogger(__name__)

def get_family(family_lines, family_type='ped'):
    """Return the families found in  a family file
    
        Args:
            family_lines (iterator): The family lines
            family_type (str): The format of the family lines
        
        Returns:
            family (Family): A ped_parser family object
    """
    family = None
    logger.info("Parsing family information")
    family_parser = FamilyParser(family_lines, family_type)
    
    families = list(family_parser.families.keys())
    
    logger.info("Found families {0}".format(', '.join(families)))
    
    if len(families) > 1:
        raise CaseError("Only one family per load can be used")
    
    family = family_parser.families[families[0]]
    
    return family



from .cli import cli as base_command

from .wipe import wipe as wipe_command
from .load import load as load_command
from .delete import delete as delete_command
from .export import export as export_command



import click

from loqusdb.log import LEVELS, init_log
from loqusdb import logger, __version__
from loqusdb.plugins import MongoAdapter

@click.group()
@click.option('-db', '--database',
                default='loqusdb',
                show_default=True,
)
@click.option('-u', '--username',
                type=str
)
@click.option('-p', '--password',
                type=str
)
@click.option('-port', '--port',
                default=27017,
                show_default=True,
                help='Specify the port where to look for the mongo database.'
)
@click.option('-h', '--host',
                default='localhost',
                show_default=True,
                help='Specify the host where to look for the mongo database.'
)
@click.option('-b', '--backend',
                default='mongo',
                show_default=True,
                type=click.Choice(['mongo',]),
                help='Specify what backend to use.'
)
@click.option('-c', '--conn_host',
                default='mongodb://',
                show_default=True,
                help='Used for testing.'
)
@click.option('-l', '--logfile',
                    type=click.Path(exists=False),
                    help=u"Path to log file. If none logging is "\
                          "printed to stderr."
)
@click.option('-v', '--verbose', count=True, default=1)
@click.version_option(__version__)
@click.pass_context
def cli(ctx, conn_host, database, username, password, port, host, verbose, 
        logfile, backend):
    """loqusdb: manage a local variant count database."""
    # configure root logger to print to STDERR
    loglevel = LEVELS.get(max(verbose,1), "INFO")
    init_log(
        logger = logger, 
        filename = logfile, 
        loglevel = loglevel
    )
    
    # mongo uri looks like:
    #mongodb://[username:password@]host1[:port1][,host2[:port2],...[,hostN[:portN]]][/[database][?options]]
    uri = None
    if username and password:
        uri = "{0}{1}:{2}@{3}:{4}/{5}".format(
              conn_host, username, password, host, port, database
              )
        logger.info('uri={0}'.format(uri))
    
    adapter = MongoAdapter()
    adapter.connect(
        host=host, 
        port=port, 
        database=database,
        uri=uri
    )
    
    ctx.obj = {}
    ctx.obj['db'] = database
    ctx.obj['user'] = username
    ctx.obj['password'] = password
    ctx.obj['port'] = port
    ctx.obj['host'] = host
    ctx.obj['adapter'] = adapter



import logging
import click
import tempfile

from datetime import datetime

from vcftoolbox import (HeaderParser, sort_variants, print_headers, 
                        print_variant)

from loqusdb import __version__

from . import base_command

logger = logging.getLogger(__name__)

@base_command.command()
@click.option('-o', '--outfile',
    type=click.File('w'),
    help='Specify the path to a file where results should be stored.'
)
@click.pass_context
def export(ctx, outfile):
    """Export the variants of a loqus db
        
        The variants are exported to a vcf file
    """
    adapter = ctx.obj['adapter']
    
    logger.info("Export the variants from {0}".format(adapter))
    nr_cases = 0
    
    for nr_cases, case in enumerate(adapter.cases()):
        nr_cases += 1
    logger.info("Found {0} cases in database".format(nr_cases))
    
    head = HeaderParser()
    head.add_fileformat("##fileformat=VCFv4.1")
    head.add_meta_line("NrCases", nr_cases)
    head.add_info("Obs", '1', 'Integer', "The number of observations for the variant")
    head.add_info("Hom", '1', 'Integer', "The number of observed homozygotes")
    head.add_version_tracking("loqusdb", __version__, datetime.now().strftime("%Y-%m-%d %H:%M"))
    
    logger.debug("Create tempfile to print variants from database")
    variants = tempfile.TemporaryFile()
    
    logger.debug("Printing headers")
    print_headers(head, outfile=outfile)
    
    try:
        for variant in adapter.get_variants():
            variant_id = variant['_id'].split('_')
            chrom = variant_id[0]
            pos = variant_id[1]
            ref = variant_id[2]
            alt = variant_id[3]
            
            observations = variant['observations']
            homozygotes = variant['homozygote']
            
            info = "Obs={0};Hom={1}".format(observations, homozygotes)
            
            variant_line = "{0}\t{1}\t.\t{2}\t{3}\t.\t.\t{4}\n".format(
                chrom, pos, ref, alt, info)
            
            variants.write(variant_line)
        
        variants.seek(0)
        for line in sort_variants(variants):
            print_variant(variant_line=line, outfile=outfile)
    finally:
        variants.close()


class BaseCaseMixin(object):
    
    def add_case(self, case):
        """Add a case to the case collection
        
            If the case exists raise error.
        
            Args:
                case (dict): A case dictionary
        """
        raise NotImplementedError
    
    def case(self):
        """Get a case from the database
    
            Search the cases with the case id
        
            Args:
                case (dict): A case dictionary
        
            Returns:
                mongo_case (dict): A mongo case dictionary
        """
        raise NotImplementedError
    
    def delete_case(self, case):
        """Delete case from the database
    
            Delete a case from the database
        
            Args:
                case (dict): A case dictionary    
        """
        raise NotImplementedError
    


# -*- coding: utf-8 -*-
import logging
import click
from datetime import datetime

from loqusdb.exceptions import CaseError
from loqusdb.utils import delete as delete_variants
from . import base_command

logger = logging.getLogger(__name__)

@base_command.command()
@click.argument('variant_file',
                    type=click.Path(exists=True),
                    metavar='<vcf_file>'
)
@click.option('-f', '--family_file',
                    type=click.Path(exists=True),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.pass_context
def delete(ctx, variant_file, family_file, family_type):
    """Delete the variants of a case."""
    if not family_file:
        logger.error("Please provide a family file")
        ctx.abort()
    
    adapter = ctx.obj['adapter']

    start_deleting = datetime.now()
    try:
        delete_variants(
            adapter=adapter, 
            variant_file=variant_file, 
            family_file=family_file, 
            family_type=family_type
        )
    except (CaseError, IOError) as error:
        logger.warning(error.message)
        ctx.abort()

    logger.info("Time to delete variants: {0}"
                .format(datetime.now() - start_deleting))



from loqusdb.utils import (delete_variants, load_variants)


def test_delete_variants(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db
    
    vcf = []
    vcf.append(het_variant)
    
    family_id = case_obj.family_id
    individuals = case_obj.individuals
    
    load_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf,
        gq_treshold=20,
    )
    
    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id]

    delete_variants(
        adapter=mongo_adapter,
        family_id=family_id, 
        individuals=individuals, 
        vcf=vcf
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant == None

def test_delete_variant(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db

    vcf = []
    vcf.append(het_variant)

    family_id_1 = case_obj.family_id
    individuals = case_obj.individuals

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id_1,
        individuals=individuals,
        vcf=vcf,
        gq_treshold=20,
    )

    mongo_variant = db.variant.find_one()
    
    assert mongo_variant['families'] == [family_id_1]

    family_id_2 = '2'

    load_variants(
        adapter=mongo_adapter,
        family_id=family_id_2,
        individuals=individuals,
        vcf=vcf,
        gq_treshold=20,
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant['families'] == [family_id_1, family_id_2]
    assert mongo_variant['observations'] == 2

    delete_variants(
        adapter=mongo_adapter,
        family_id=family_id_2,
        individuals=individuals,
        vcf=vcf
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant['families'] == [family_id_1]
    assert mongo_variant['observations'] == 1

def test_delete_non_existing_variant(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    db = mongo_adapter.db

    vcf = []
    vcf.append(het_variant)
    family_id = case_obj.family_id
    individuals = case_obj.individuals

    delete_variants(
        adapter=mongo_adapter,
        family_id=family_id,
        individuals=individuals,
        vcf=vcf
    )

    mongo_variant = db.variant.find_one()

    assert mongo_variant == None



import logging

logger = logging.getLogger(__name__)

class Base(object):
    """The base class of an adapter"""
    
    def connect(self):
        """Establish a connection to databse"""
        raise NotImplementedError
    
    def wipe_db(self):
        """Wipe the databsae"""
        raise NotImplementedError


from loqusdb.plugins import MongoAdapter

class FlaskApp(object):
    """Mock a flask app"""
    def __init__(self):
        super(FlaskApp, self).__init__()
        self.config = {}

def test_init_app(mongo_client):
    app = FlaskApp()
    app.config['LOQUSDB_URI'] = "mongodb://localhost"
    adapter = MongoAdapter()
    adapter.init_app(app)
    assert True


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Based on https://github.com/pypa/sampleproject/blob/master/setup.py."""
from __future__ import unicode_literals
# To use a consistent encoding
import codecs
import os
from setuptools import setup, find_packages
import sys

# Shortcut for building/publishing to Pypi
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist bdist_wheel upload')
    sys.exit()


def parse_reqs(req_path='./requirements.txt'):
    """Recursively parse requirements from nested pip files."""
    install_requires = []
    with codecs.open(req_path, 'r') as handle:
        # remove comments and empty lines
        lines = (line.strip() for line in handle
                 if line.strip() and not line.startswith('#'))

        for line in lines:
            # check for nested requirements files
            if line.startswith('-r'):
                # recursively call this function
                install_requires += parse_reqs(req_path=line[3:])

            else:
                # add the line as a new requirement
                install_requires.append(line)

    return install_requires

# For making things look nice on pypi:
try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, RuntimeError):
    long_description = 'Tool for annotating patterns of genetic inheritance in Variant Call Format (VCF) files.'


# with open('README.txt') as file:
#     long_description = file.read()

setup(name='loqusdb',
    version='0.4',
    description='Store frequencies of vcf variants in a mongodb',
    author = 'Mans Magnusson',
    author_email = 'mans.magnusson@scilifelab.se',
    url = 'http://github.com/moonso/loqusdb',
    license = 'MIT License',
    install_requires=parse_reqs(),
    
    packages=find_packages(exclude=('tests*', 'docs', 'examples')),
    
    entry_points=dict(
        console_scripts=[
        "loqusdb = loqusdb.__main__:base_command",
        ],
    ),
    keywords = ['vcf', 'variants'],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Unix",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    long_description = long_description,
)


import logging

from vcftoolbox import (Genotype)

from loqusdb.exceptions import CaseError

logger = logging.getLogger(__name__)

# These are coordinate for the pseudo autosomal regions in GRCh37
PAR = {
    'Y': [[10001, 2649520], [59034050, 59373566]],
    'X': [[60001, 2699520], [154931044, 155270560]]
}

def check_par(chrom, pos):
    """Check if a coordinate is in the PAR region
    
        Args:
            chrom(str)
            pos(int)
    
        Returns:
            par(bool)
    """
    par = False
    
    for interval in PAR.get(chrom,[]):
        if (pos >= interval[0] and pos <= interval[1]):
            par = True
    
    return par
    

def get_formated_variant(variant, individuals, family_id, gq_treshold=None):
    """Return a formated variant line
    
        Take a vcf formated variant line and return a dictionary with the
        relevant information.
    
        If criterias are not fullfilled, eg. variant have no gt call or quality
        is below gq treshold then an empty dictionary is returned.
        
        Args:
            variant (dict): A variant dictionary
            individuals (list[str]): A list with individual ids
            family_id (str): The family id
        
        Return:
            formated_variant (dict): A variant dictionary
    """
    gq_treshold = gq_treshold or 20
    
    chrom = variant['CHROM'].lstrip('chr')
    pos = int(variant['POS'])
    ref = variant['REF']
    alt = variant['ALT']

    formated_variant = {}

    if ',' in alt:
        raise Exception("Multi allele calls are not allowed.")

    format_field = variant['FORMAT'].split(':')

    found_variant = False
    found_homozygote = False
    found_hemizygote = False

    for ind_id in individuals:
        ind_obj = individuals[ind_id]
        
        if ind_id in variant:
            raw_gt_call = variant[ind_id]
        else:
            raise CaseError("Individual {0} from ped does not exist in"\
                            " vcf".format(ind_id))

        gt_call = dict(zip(
                format_field,
                raw_gt_call.split(':'))
                )

        genotype = Genotype(**gt_call)
        if genotype.genotype_quality >= gq_treshold:
            if genotype.has_variant:
                logger.debug("Found variant in affected")
                found_variant = True
            
                # If variant in X or Y and individual is male,
                # we need to check hemizygosity
                if chrom in ['X','Y'] and ind_obj.sex == 1:
                    if not check_par(chrom, pos):
                        logger.debug("Found hemizygous variant")
                        found_hemizygote = True
                
                if genotype.homo_alt:
                    logger.debug("Found homozygote alternative variant")
                    found_homozygote = True

    if found_variant:
        formated_variant['_id'] = '_'.join([chrom, str(pos), ref, alt])
        formated_variant['chrom'] = chrom
        formated_variant['pos'] = pos
        formated_variant['ref'] = ref
        formated_variant['alt'] = alt
        formated_variant['homozygote'] = 0
        formated_variant['hemizygote'] = 0

        if found_hemizygote:
            formated_variant['hemizygote'] = 1
        elif found_homozygote:
            formated_variant['homozygote'] = 1
        
        if family_id:
            formated_variant['family_id'] = family_id
    
    return formated_variant



from loqusdb.commands import base_command

from click.testing import CliRunner

def test_base_command():
    runner = CliRunner()
    result = runner.invoke(base_command, [])
    
    assert result.exit_code == 0

class TestLoadCommand:
    
    # def test_load_command(self, mongo_client, vcf_path, ped_path):
    #     runner = CliRunner()
    #     command = ['-c', 'mongomock://', '--database', 'test', 'load', vcf_path, '-f', ped_path]
    #     result = runner.invoke(base_command, command)
    #
    #     assert result.exit_code == 0

    def test_load_command_no_ped(self, mongo_client, vcf_path):
        runner = CliRunner()
        command = ['-c', 'mongomock://', '--database', 'test', 'load', vcf_path]
        result = runner.invoke(base_command, command)
        
        assert result.exit_code == 1
    
# def test_delete_command(mongo_client, vcf_path, ped_path):
#     runner = CliRunner()
#
#     load_command = ['-c', 'mongomock://', '--database', 'test', 'load', vcf_path, '-f', ped_path]
#     runner.invoke(base_command, load_command)
#
#     delete_command = ['-c', 'mongomock://', '--database', 'test', 'delete', vcf_path, '-f', ped_path]
#     result = runner.invoke(base_command, delete_command)
#
#     assert result.exit_code == 0
#
#
# def test_wipe_command(mongo_client, vcf_path, ped_path):
#     runner = CliRunner()
#
#     load_command = ['-c', 'mongomock://', '--database', 'test', 'load', vcf_path, '-f', ped_path]
#     runner.invoke(base_command, load_command)
#
#     wipe_command = ['-c', 'mongomock://', '--database', 'test', 'wipe', '--yes']
#     result = runner.invoke(base_command, wipe_command)
#
#     assert result.exit_code == 0




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
loqusdb.__main__
~~~~~~~~~~~~~~~~~~~~~

The main entry point for the command line interface.

Invoke as ``loqusdb`` (if installed)
or ``python -m loqusdb`` (no install required).
"""
import sys

from loqusdb.commands import base_command


if __name__ == '__main__':
    # exit using whatever exit code the CLI returned
    sys.exit(base_command())
